﻿// migration from java to .NET by SimpleDB# group, 
// http://cs.tju.edu.cn/faculty/zhangkl/projects/simpledbsharp/index.htm

using simpledb.file;

namespace simpledb.buffer
{
    /// <summary>
    /// Manages the pinning and unpinning of buffers to blocks.
    /// </summary>
    /// <author>Edward Sciore</author>
    internal class BasicBufferMgr
    {
        private Buffer[] bufferpool;
        private int numAvailable;
        private object threadLock = new object();

        /// <summary>
        /// Creates a buffer manager having the specified number 
        /// of buffer slots.
        /// </summary>
        /// <param name="numbuffs">the number of buffer slots to allocate</param>      
        internal BasicBufferMgr(int numbuffs)
        {
            bufferpool = new Buffer[numbuffs];
            numAvailable = numbuffs;
            for (int i = 0; i < numbuffs; i++)
                bufferpool[i] = new Buffer();
        }

        /// <summary>
        /// Flushes the dirty buffers modified by the specified transaction.
        /// </summary>
        /// <param name="txnum">the transaction's id number</param>
        internal void flushAll(int txnum)
        {
            lock (threadLock)
            {
                foreach (Buffer buff in bufferpool)
                {
                    if (buff.isModifiedBy(txnum))
                        buff.flush();
                }
            }
        }

        /// <summary>
        /// Pins a buffer to the specified block. 
        /// If there is already a buffer assigned to that block
        /// then that buffer is used;  
        /// otherwise, an unpinned buffer from the pool is chosen.
        /// Returns a null value if there are no available buffers.
        /// </summary>
        /// <param name="blk">a reference to a disk block</param>  
        /// <returns>the pinned buffer</returns>
        internal Buffer pin(Block blk)
        {
            lock (threadLock)
            {
                Buffer buff = findExistingBuffer(blk);
                if (buff == null)
                {
                    buff = chooseUnpinnedBuffer();
                    if (buff == null)
                        return null;
                    buff.assignToBlock(blk);
                }
                if (!buff.isPinned())
                    numAvailable--;
                buff.pin();
                return buff;
            }
        }

        /// <summary>
        /// Allocates a new block in the specified file, and
        /// pins a buffer to it. 
        /// Returns null (without allocating the block) if 
        /// there are no available buffers.
        /// </summary>
        /// <param name="filename">the name of the file</param>  
        /// <param name="fmtr">a pageformatter object, used to format the new block</param>  
        /// <returns>the pinned buffer</returns>  
        internal Buffer pinNew(string filename, PageFormatter fmtr)
        {
            lock (threadLock)
            {
                Buffer buff = chooseUnpinnedBuffer();
                if (buff == null)
                    return null;
                buff.assignToNew(filename, fmtr);
                numAvailable--;
                buff.pin();
                return buff;
            }
        }

        /// <summary>
        /// Unpins the specified buffer.
        /// </summary>
        /// <param name="buff">the buffer to be unpinned</param>
        internal void unpin(Buffer buff)
        {
            lock (threadLock)
            {
                buff.unpin();
                if (!buff.isPinned())
                    numAvailable++;
            }
        }

        /// <summary>
        /// Returns the number of available (i.e. unpinned) buffers.
        /// </summary>
        /// <returns>the number of available buffers</returns>
        internal int available()
        {
            return numAvailable;
        }

        private Buffer findExistingBuffer(Block blk)
        {
            foreach (Buffer buff in bufferpool)
            {
                Block b = buff.block();
                if (b != null && b.Equals(blk))
                    return buff;
            }
            return null;
        }

        private Buffer chooseUnpinnedBuffer()
        {
            foreach (Buffer buff in bufferpool)
                if (!buff.isPinned())
                    return buff;
            return null;
        }

    }
}
