﻿// migration from java to .NET by SimpleDB# group, 
// http://cs.tju.edu.cn/faculty/zhangkl/projects/simpledbsharp/index.htm

using System;
using simpledb.file;
using System.Threading;

namespace simpledb.buffer
{
    /// <summary>
    /// The publicly-accessible buffer manager.
    /// A buffer manager wraps a basic buffer manager, and
    /// provides the same methods. The difference is that
    /// the methods <see cref="pin"/> and 
    /// <see cref="pinNew"/> 
    /// will never return null.
    /// If no buffers are currently available, then the
    /// calling thread will be placed on a waiting list.
    /// The waiting threads are removed from the list when 
    /// a buffer becomes available.
    /// If a thread has been waiting for a buffer for an
    /// excessive amount of time (currently, 10 seconds)
    /// then a <see cref="BufferAbortException"/> is thrown.
    /// </summary>
    /// <author>Edward Sciore</author>
    public class BufferMgr
    {
        private const int MAX_TIME = 10000; // 10 seconds
        private BasicBufferMgr bbm;
        private object threadLock = new object();

        /// <summary>
        /// Creates a new buffer manager having 8the specified 
        /// number of buffers.
        /// </summary>
        /// <param name="numbuffers">the number of buffer slots to allocate</param>
        public BufferMgr(int numbuffers)
        {
            bbm = new BasicBufferMgr(numbuffers);
        }

        /// <summary>
        /// Pins a buffer to the specified block,potentially waiting
        /// until a buffer becomes available.
        /// If no buffer becomes available within a fixed 
        /// time period, then a <see cref="BufferAbortException"/> is thrown.
        /// </summary>
        /// <param name="blk">a reference to a disk block</param>
        /// <returns>the buffer pinned to that block</returns>
        public Buffer pin(Block blk)
        {
            lock (threadLock)
            {
                try
                {
                    long timestamp = DateTime.Now.Ticks;
                    Buffer buff = bbm.pin(blk);
                    while (buff == null && !waitingTooLong(timestamp))
                    {
                        Monitor.Wait(threadLock, MAX_TIME);
                        buff = bbm.pin(blk);
                    }
                    if (buff == null)
                        throw new BufferAbortException();
                    return buff;
                }
                catch (ThreadInterruptedException)
                {
                    throw new BufferAbortException();
                }
            }
        }

        /// <summary>
        /// Pins a buffer to a new block in the specified file, 
        /// potentially waiting until a buffer becomes available.
        /// If no buffer becomes available within a fixed 
        /// time period, then a <see cref="BufferAbortException"/> is thrown.
        /// </summary>
        /// <param name="filename">the name of the file</param>  
        /// <param name="fmtr">the formatter used to initialize the page</param>  
        /// <returns>the buffer pinned to that block</returns> 
        public Buffer pinNew(string filename, PageFormatter fmtr)
        {
            lock (threadLock)
            {
                try
                {
                    long timestamp = DateTime.Now.Ticks;
                    Buffer buff = bbm.pinNew(filename, fmtr);
                    while (buff == null && !waitingTooLong(timestamp))
                    {
                        Monitor.Wait(threadLock, MAX_TIME);
                        buff = bbm.pinNew(filename, fmtr);
                    }
                    if (buff == null)
                        throw new BufferAbortException();
                    return buff;
                }
                catch (ThreadInterruptedException)
                {
                    throw new BufferAbortException();
                }
            }
        }

        /// <summary>
        /// Unpins the specified buffer. 
        /// If the buffer's pin count becomes 0,
        /// then the threads on the wait list are notified.
        /// </summary>
        /// <param name="buff">the buffer to be unpinned.</param>
        public void unpin(Buffer buff)
        {
            lock (threadLock)
            {
                bbm.unpin(buff);
                if (!buff.isPinned())
                    Monitor.PulseAll(threadLock);
            }
        }

        /// <summary>
        /// Flushes the dirty buffers modified by the specified transaction.
        /// </summary>
        /// <param name="txnum">the transaction's id number</param>
        public void flushAll(int txnum)
        {
            bbm.flushAll(txnum);
        }

        /// <summary>
        /// Returns the number of available (ie unpinned) buffers.
        /// </summary>
        /// <returns>the number of available buffers</returns>
        public int available()
        {
            return bbm.available();
        }

        private bool waitingTooLong(long starttime)
        {
            return (DateTime.Now.Ticks - starttime) / 10000 > MAX_TIME;
        }
    }
}
