﻿// migration from java to .NET by SimpleDB# group, 
// http://cs.tju.edu.cn/faculty/zhangkl/projects/simpledbsharp/index.htm

using simpledb.server;

namespace simpledb.file
{
    /// <summary>
    /// The contents of a disk block in memory.
    /// A page is treated as an array of BLOCK_SIZE bytes.
    /// There are methods to get/set values into this array,
    /// and to read/write the contents of this array to a disk block.
    /// </summary>
    /// <example>
    /// For an example of how to use Page and 
    /// <see cref="Block"/> objects, 
    /// consider the following code fragment.  
    /// The first portion increments the integer at offset 792 of block 6 of file junk.  
    /// The second portion stores the string "hello" at offset 20 of a page, 
    /// and then appends it to a new block of the file.  
    /// It then reads that block into another page 
    /// and extracts the value "hello" into variable s.
    /// <code>
    /// Page p1 = new Page();
    /// Block blk = new Block("junk", 6);
    /// p1.read(blk);
    /// int n = p1.getInt(792);
    /// p1.setInt(792, n+1);
    /// p1.write(blk);
    /// Page p2 = new Page();
    /// p2.setString(20, "hello");
    /// blk = p2.append("junk");
    /// Page p3 = new Page();
    /// p3.read(blk);
    /// string s = p3.getString(20);
    /// </code>
    /// </example>
    /// <author>Edward Sciore</author>
    public class Page
    {
        /// <summary>
        /// The number of bytes in a block.
        /// This value is set unreasonably low, so that it is easier
        /// to create and test databases having a lot of blocks.
        /// A more realistic value would be 4K.
        /// </summary>
        public const int BLOCK_SIZE = 512;

        /// <summary>
        /// The size of an integer in bytes.
        /// This value is almost certainly 4, but it is
        /// a good idea to encode this value as a constant.         
        /// </summary>
        public const int INT_SIZE = sizeof(int) / sizeof(byte);

        /// <summary>
        /// The maximum size, in bytes, of a string of length n.
        /// A string is represented as the encoding of its characters,
        /// preceded by an integer denoting the number of bytes in this encoding.
        /// The .Net uses the Unicode encoding, then each char
        /// is stored in two bytes, so a string of n characters
        /// has a size of 4+n*2 bytes.
        /// </summary>
        /// <param name="n">the size of the string</param>
        /// <returns>the maximum number of bytes required to store a string of size n</returns>
        public static int STR_SIZE(int n)
        {
            return INT_SIZE + n * sizeof(char);
        }

        private byte[] contents = new byte[BLOCK_SIZE];
        private FileMgr filemgr = SimpleDB.fileMgr();
        private object threadLock = new object();

        /// <summary>
        /// Populates the page with the contents of the specified disk block.
        /// </summary>
        /// <param name="blk">a reference to a disk block.</param>
        public void read(Block blk)
        {
            lock (threadLock)
            {
                filemgr.read(blk, contents);
            }
        }

        /// <summary>
        /// Writes the contents of the page to the specified disk block.
        /// </summary>
        /// <param name="blk">a reference to a disk block.</param>
        public void write(Block blk)
        {
            lock (threadLock)
            {
                filemgr.write(blk, contents);
            }
        }

        /// <summary>
        /// Appends the contents of the page to the specified file.
        /// </summary>
        /// <param name="filename">the name of the file</param>
        /// <returns>the reference to the newly-created disk block</returns>
        public Block append(string filename)
        {
            lock (threadLock)
            {
                return filemgr.append(filename, contents);
            }
        }

        /// <summary>
        /// Returns the integer value at a specified offset of the page.
        /// If an integer was not stored at that location, 
        /// the behavior of the method is unpredictable.
        /// </summary>
        /// <param name="offset">the byte offset within the page</param>
        /// <returns>the integer value at that offset</returns>
        public int getInt(int offset)
        {
            lock (threadLock)
            {
                return (int)(contents[offset] | contents[offset + 1] << 8 | contents[offset + 2] << 16 | contents[offset + 3] << 24);
            }
        }

        /// <summary>
        /// Writes a integer to the specified offset on the page.
        /// </summary>
        /// <param name="offset">the byte offset within the page</param>
        /// <param name="val">the integer to be written to the page</param>
        public void setInt(int offset, int val)
        {
            lock (threadLock)
            {
                // Little-Endian
                contents[offset] = (byte)(val);
                contents[offset + 1] = (byte)(val >> 8);
                contents[offset + 2] = (byte)(val >> 16);
                contents[offset + 3] = (byte)(val >> 24);
            }
        }

        /// <summary>
        /// Returns the string value at a specified offset of the page.
        /// If an string was not stored at that location, 
        /// the behavior of the method is unpredictable.
        /// </summary>
        /// <param name="offset">the byte offset within the page</param>
        /// <returns>the integer value at that offset</returns>
        public string getString(int offset)
        {
            lock (threadLock)
            {
                int len = getInt(offset);
                byte[] byteval = new byte[len];
                for (int i = 0; i < len; i++)
                    byteval[i] = contents[offset + 4 + i];
                return System.Text.Encoding.Default.GetString(byteval);
            }
        }

        /// <summary>
        /// Writes a string to the specified offset on the page.
        /// </summary>
        /// <param name="offset">the byte offset within the page</param>
        /// <param name="val">the string to be written to the page</param>
        public void setString(int offset, string val)
        {
            lock (threadLock)
            {
                byte[] byteval = System.Text.Encoding.Default.GetBytes(val);
                setInt(offset, byteval.Length);
                for (int i = 0; i < byteval.Length; i++)
                    contents[offset + 4 + i] = byteval[i];
            }
        }

    }
}
