// migration from java to .NET by SimpleDB# group, 
// http://cs.tju.edu.cn/faculty/zhangkl/projects/simpledbsharp/index.htm

using simpledb.tx;
using simpledb.record;
using simpledb.query;

namespace simpledb.index.hash
{
    /// <summary>
    /// A static hash implementation of the Index interface.
    /// A fixed number of buckets is allocated (currently, 100),
    /// and each bucket is implemented as a file of index records.
    /// <author>Edward Sciore</author> 
    /// </summary>
    public class HashIndex : Index
    {
        private const int NUM_BUCKETS = 100;
        private string idxname;
        private Schema sch;
        private Transaction tx;
        private Constant searchkey = null;
        private TableScan ts = null;

        /// <summary>
        /// Opens a hash index for the specified index
        /// </summary>
        /// <param name="idxname">the name of the index</param>
        /// <param name="sch">the schema of the index records</param>
        /// <param name="tx">the calling transaction</param>
        public HashIndex(string idxname, Schema sch, Transaction tx)
        {
            this.idxname = idxname;
            this.sch = sch;
            this.tx = tx;
        }

        /// <summary>
        /// Positions the index before the first index record
        /// having the specified search key.
        /// The method hashes the search key to determine the bucket,
        /// and then opens a table scan on the file
        /// corresponding to the bucket.
        /// The table scan for the previous bucket (if any) is closed.
        /// </summary>
        /// <seealso cref="simpledb.index.Index.beforeFirst"/>        
        public void beforeFirst(Constant searchkey)
        {
            close();
            this.searchkey = searchkey;
            int bucket = searchkey.GetHashCode() % NUM_BUCKETS;
            string tblname = idxname + bucket;
            TableInfo ri = new TableInfo(tblname, sch);
            ts = new TableScan(ri, tx);
        }

        /// <summary>
        /// Moves to the next record having the search key.
        /// The method loops through the table scan for the bucket,
        /// looking for a matching record, and returning false
        /// if there are no more such records.
        /// </summary>
        /// <seealso cref="simpledb.index.Index.next"/>
        public bool next()
        {
            while (ts.next())
            {
                if (ts.getVal("dataval").Equals(searchkey))
                    return true;
            }
            return false;
        }

        /// <summary>
        /// Retrieves the dataRID from the current record
        /// in the table scan for the bucket.
        /// </summary>
        /// <seealso cref="simpledb.index.Index.getDataRid"/>
        public RID getDataRid()
        {
            int blknum = ts.getInt("block");
            int id = ts.getInt("id");
            return new RID(blknum, id);
        }

        /// <summary>
        /// Inserts a new record into the table scan for the bucket. 
        /// </summary>
        /// <seealso cref="simpledb.index.Index.insert"/> 
        public void insert(Constant val, RID rid)
        {
            beforeFirst(val);
            ts.insert();
            ts.setInt("block", rid.blockNumber());
            ts.setInt("id", rid.Id());
            ts.setVal("dataval", val);
        }

        /// <summary>
        /// Deletes the specified record from the table scan for
        /// the bucket.  The method starts at the beginning of the
        /// scan, and loops through the records until the
        /// specified record is found.
        /// </summary>
        /// <seealso cref="simpledb.index.Index.delete"/>
        public void delete(Constant val, RID rid)
        {
            beforeFirst(val);
            while (next())
            {
                if (getDataRid().Equals(rid))
                {
                    ts.delete();
                    return;
                }
            }
        }

        /// <summary>
        /// Closes the index by closing the current table scan.
        /// </summary>
        /// <seealso cref="simpledb.index.Index.close"/> 
        public void close()
        {
            if (ts != null)
                ts.close();
        }

        /// <summary>
        /// Returns the cost of searching an index file having the
        /// specified number of blocks.
        /// The method assumes that all buckets are about the
        /// same size, and so the cost is simply the size of
        /// the bucket. 
        /// </summary>
        /// <param name="numblocks">the number of blocks of index records</param>
        /// <param name="rpb">the number of records per block</param>
        /// <returns>the cost of traversing the index</returns>
        public static int searchCost(int numblocks, int rpb)
        {
            return numblocks / NUM_BUCKETS;
        }
    }
}
