// migration from java to .NET by SimpleDB# group, 
// http://cs.tju.edu.cn/faculty/zhangkl/projects/simpledbsharp/index.htm

using System.Collections.Generic;
using simpledb.record;
using simpledb.tx;
using simpledb.metadata;
using simpledb.parse;
using simpledb.query;
using simpledb.planner;
using simpledb.server;

namespace simpledb.index.planner
{
    /// <summary>
    /// A modification of the basic update planner.
    /// It dispatches each update statement to the corresponding
    /// index planner.
    /// <author>Edward Sciore</author> 
    /// </summary>
    class IndexUpdatePlanner : UpdatePlanner
    {
        public int executeInsert(InsertData data, Transaction tx)
        {
            string tblname = data.tableName();
            Plan plan = new TablePlan(tblname, tx);

            // first, insert the record
            UpdateScan s = (UpdateScan)plan.open();
            s.insert();
            RID rid = s.getRid();

            // then modify each field, inserting an index record if appropriate
            Dictionary<string, IndexInfo> indexes = SimpleDB.mdMgr().getIndexInfo(tblname, tx);
            IEnumerator<Constant> valIter = data.vals().GetEnumerator();
            foreach (string fldname in data.fields())
            {
                valIter.MoveNext();
                Constant val = valIter.Current;
                s.setVal(fldname, val);

                if (indexes.ContainsKey(fldname))
                {
                    IndexInfo ii = (IndexInfo)indexes[fldname];
                    Index idx = ii.open();
                    idx.insert(val, rid);
                    idx.close();
                }
            }
            s.close();
            return 1;
        }

        public int executeDelete(DeleteData data, Transaction tx)
        {
            string tblname = data.tableName();
            Plan p = new TablePlan(tblname, tx);
            p = new SelectPlan(p, data.pred());
            Dictionary<string, IndexInfo> indexes = SimpleDB.mdMgr().getIndexInfo(tblname, tx);

            UpdateScan s = (UpdateScan)p.open();
            int count = 0;
            while (s.next())
            {
                // first, delete the record's RID from every index
                RID rid = s.getRid();
                foreach (string fldname in indexes.Keys)
                {
                    Constant val = s.getVal(fldname);
                    Index idx = ((IndexInfo)indexes[fldname]).open();
                    idx.delete(val, rid);
                    idx.close();
                }
                // then delete the record
                s.delete();
                count++;
            }
            s.close();
            return count;
        }

        public int executeModify(ModifyData data, Transaction tx)
        {
            string tblname = data.tableName();
            string fldname = data.targetField();
            Plan p = new TablePlan(tblname, tx);
            p = new SelectPlan(p, data.pred());

            Dictionary<string, IndexInfo> indexes = SimpleDB.mdMgr().getIndexInfo(tblname, tx);
            IndexInfo ii = null;
            if (indexes.ContainsKey(fldname))
                ii = indexes[fldname];
            Index idx = (ii == null) ? null : ii.open();

            UpdateScan s = (UpdateScan)p.open();
            int count = 0;
            while (s.next())
            {
                // first, update the record
                Constant newval = data.newValue().evaluate(s);
                Constant olddval = s.getVal(fldname);
                s.setVal(data.targetField(), newval);

                // then update the appropriate index, if it exists
                if (idx != null)
                {
                    RID rid = s.getRid();
                    idx.delete(olddval, rid);
                    idx.insert(newval, rid);
                }
                count++;
            }
            if (idx != null)
                idx.close();
            s.close();
            return count;
        }

        public int executeCreateTable(CreateTableData data, Transaction tx)
        {
            SimpleDB.mdMgr().createTable(data.tableName(), data.newSchema(), tx);
            return 0;
        }

        public int executeCreateView(CreateViewData data, Transaction tx)
        {
            SimpleDB.mdMgr().createView(data.viewName(), data.viewDef(), tx);
            return 0;
        }

        public int executeCreateIndex(CreateIndexData data, Transaction tx)
        {
            SimpleDB.mdMgr().createIndex(data.indexName(), data.tableName(), data.fieldName(), tx);
            return 0;
        }
    }
}
