﻿// migration from java to .NET by SimpleDB# group, 
// http://cs.tju.edu.cn/faculty/zhangkl/projects/simpledbsharp/index.htm

using System.Collections.Generic;
using simpledb.tx;
using simpledb.record;
using simpledb.query;

namespace simpledb.materialize
{
    /// <summary>
    /// The Plan class for the <i>groupby</i> operator.
    /// </summary>
    /// <author>Edward Sciore</author>
    class GroupByPlan
    {
        private Plan p;
        private List<string> groupfields;
        private List<AggregationFn> aggfns;
        private Schema sch = new Schema();

        /// <summary>
        ///  Creates a groupby plan for the underlying query.
        /// The grouping is determined by the specified
        /// collection of group fields,
        /// and the aggregation is computed by the
        /// specified collection of aggregation functions.
        /// </summary>
        /// <param name="p">a plan for the underlying query</param>  
        /// <param name="groupfields "> the group fields</param>
        /// <param name="aggfns"> the aggregation functions</param> 
        /// <param name="tx">the calling transaction</param>  
        public GroupByPlan(Plan p, List<string> groupfields, List<AggregationFn> aggfns, Transaction tx)
        {
            List<string> grouplist = new List<string>();
            grouplist.AddRange(groupfields);
            this.p = new SortPlan(p, grouplist, tx);
            this.groupfields = groupfields;
            this.aggfns = aggfns;
            foreach (string fldname in groupfields)
                sch.add(fldname, p.schema());
            foreach (AggregationFn fn in aggfns)
                sch.addIntField(fn.fieldName());
        }

        /// <summary>
        ///  This method opens a sort plan for the specified plan.
        /// The sort plan ensures that the underlying records
        /// will be appropriately grouped.
        /// </summary>
        /// <seealso cref="simpledb.query.Plan.open"/>  
        public Scan open()
        {
            Scan s = p.open();
            return (Scan)new GroupByScan(s, groupfields, aggfns);
        }

        /// <summary>
        ///  Returns the number of blocks required to
        /// compute the aggregation,
        /// which is one pass through the sorted table.
        /// It does <i>not</i> include the one-time cost
        /// of materializing and sorting the records.
        /// </summary>
        /// <seealso cref="simpledb.query.Plan.blocksAccessed"/> 
        public int blocksAccessed()
        {
            return p.blocksAccessed();
        }

        /// <summary>
        ///  Returns the number of groups.  Assuming equal distribution,
        /// this is the product of the distinct values
        /// for each grouping field.
        /// </summary>
        /// <seealso cref="simpledb.query.Plan.recordsOutput"/> 
        public int recordsOutput()
        {
            int numgroups = 1;
            foreach (string fldname in groupfields)
                numgroups *= p.distinctValues(fldname);
            return numgroups;
        }

        /// <summary>
        /// Returns the number of distinct values for the
        /// specified field.  If the field is a grouping field,
        /// then the number of distinct values is the same
        /// as in the underlying query.
        /// If the field is an aggregate field, then we
        /// assume that all values are distinct.
        /// </summary>
        /// <seealso cref="simpledb.query.Plan.distinctValues"/> 
        public int distinctValues(string fldname)
        {
            if (p.schema().hasField(fldname))
                return p.distinctValues(fldname);
            else
                return recordsOutput();
        }

        /// <summary>
        ///  Returns the schema of the output table.
        /// The schema consists of the group fields,
        /// plus one field for each aggregation function.
        /// </summary>
        /// <seealso cref="simpledb.query.Plan.schema"/> 
        public Schema schema()
        {
            return sch;
        }
    }
}
