// migration from java to .NET by SimpleDB# group, 
// http://cs.tju.edu.cn/faculty/zhangkl/projects/simpledbsharp/index.htm

using System.Collections.Generic;
using simpledb.record;
using simpledb.tx;

namespace simpledb.metadata
{
    /// <summary>
    /// The index manager.
    /// The index manager has similar functionalty to the table manager. 
    /// </summary>
    /// <author>Edward Sciore</author>
    class IndexMgr
    {
        private TableInfo ti;

        /// <summary>
        /// Creates the index manager.
        /// This constructor is called during system startup.
        /// If the database is new, then the <code>idxcat</code> table is created.
        /// </summary>
        /// <param name="isnew">indicates whether this is a new database</param>
        /// <param name="tblMgr"></param>
        /// <param name="tx">the system startup transaction</param>
        public IndexMgr(bool isnew, TableMgr tblMgr, Transaction tx)
        {
            if (isnew)
            {
                Schema sch = new Schema();
                sch.addStringField("indexname", TableMgr.MAX_NAME);
                sch.addStringField("tablename", TableMgr.MAX_NAME);
                sch.addStringField("fieldname", TableMgr.MAX_NAME);
                tblMgr.createTable("idxcat", sch, tx);
            }
            ti = tblMgr.getTableInfo("idxcat", tx);
        }

        /// <summary>
        /// Creates an index of the specified type for the specified field.
        /// A unique ID is assigned to this index, and its information
        /// is stored in the idxcat table.
        /// </summary>
        /// <param name="idxname">the name of the index</param>
        /// <param name="tblname">the name of the indexed table</param>
        /// <param name="fldname">the name of the indexed field</param>
        /// <param name="tx">the calling transaction</param>
        public void createIndex(string idxname, string tblname, string fldname, Transaction tx)
        {
            RecordFile rf = new RecordFile(ti, tx);
            rf.insert();
            rf.setString("indexname", idxname);
            rf.setString("tablename", tblname);
            rf.setString("fieldname", fldname);
            rf.close();
        }

        /// <summary>
        /// Returns a map containing the index info for all indexes
        /// on the specified table.
        /// </summary>
        /// <param name="tblname">the name of the table</param>
        /// <param name="tx">the calling transaction</param>
        /// <returns>a map of IndexInfo objects, keyed by their field names</returns>
        public Dictionary<string, IndexInfo> getIndexInfo(string tblname, Transaction tx)
        {
            Dictionary<string, IndexInfo> result = new Dictionary<string, IndexInfo>();
            RecordFile rf = new RecordFile(ti, tx);
            while (rf.next())
            {
                if (rf.getString("tablename").Equals(tblname))
                {
                    string idxname = rf.getString("indexname");
                    string fldname = rf.getString("fieldname");
                    IndexInfo ii = new IndexInfo(idxname, tblname, fldname, tx);
                    result.Add(fldname, ii);
                }
            }
            rf.close();
            return result;
        }
    }
}
