// migration from java to .NET by SimpleDB# group, 
// http://cs.tju.edu.cn/faculty/zhangkl/projects/simpledbsharp/index.htm

using System.Collections.Generic;
using simpledb.tx;
using simpledb.record;

namespace simpledb.metadata
{
    /// <summary>
    /// Metadata manager
    /// </summary>
    public class MetadataMgr
    {
        private static TableMgr tblmgr;
        private static ViewMgr viewmgr;
        private static StatMgr statmgr;
        private static IndexMgr idxmgr;

        /// <summary>
        /// Initializes a new instance of the <see cref="MetadataMgr"/> class.
        /// </summary>
        /// <param name="isnew">Is a new database created?</param>
        /// <param name="tx">The transaction.</param>
        public MetadataMgr(bool isnew, Transaction tx)
        {
            tblmgr = new TableMgr(isnew, tx);
            viewmgr = new ViewMgr(isnew, tblmgr, tx);
            statmgr = new StatMgr(tblmgr, tx);
            idxmgr = new IndexMgr(isnew, tblmgr, tx);
        }

        /// <summary>
        /// Creates a new table having the specified name and schema.
        /// </summary>
        /// <param name="tblname">the name of the new table</param>
        /// <param name="sch">the table's schema</param>
        /// <param name="tx">the transaction creating the table</param>
        public void createTable(string tblname, Schema sch, Transaction tx)
        {
            tblmgr.createTable(tblname, sch, tx);
        }

        /// <summary>
        /// Retrieves the metadata for the specified table
        /// out of the catalog.
        /// </summary>
        /// <param name="tblname">the name of the table</param>
        /// <param name="tx">the transaction</param>
        /// <returns>the table's stored metadata</returns>
        public TableInfo getTableInfo(string tblname, Transaction tx)
        {
            return tblmgr.getTableInfo(tblname, tx);
        }

        /// <summary>
        /// Creates a view.
        /// </summary>
        /// <param name="viewname">The view name.</param>
        /// <param name="viewdef">The view definition.</param>
        /// <param name="tx">The transaction.</param>
        public void createView(string viewname, string viewdef, Transaction tx)
        {
            viewmgr.createView(viewname, viewdef, tx);
        }

        /// <summary>
        /// Gets the view definition.
        /// </summary>
        /// <param name="viewname">The view name.</param>
        /// <param name="tx">The transaction.</param>
        /// <returns>The view definition</returns>
        public string getViewDef(string viewname, Transaction tx)
        {
            return viewmgr.getViewDef(viewname, tx);
        }

        /// <summary>
        /// Creates an index of the specified type for the specified field.
        /// A unique ID is assigned to this index, and its information
        /// is stored in the idxcat table.
        /// </summary>
        /// <param name="idxname">the name of the index</param>
        /// <param name="tblname">the name of the indexed table</param>
        /// <param name="fldname">the name of the indexed field</param>
        /// <param name="tx">the calling transaction</param>
        public void createIndex(string idxname, string tblname, string fldname, Transaction tx)
        {
            idxmgr.createIndex(idxname, tblname, fldname, tx);
        }

        /// <summary>
        /// Returns a dictionary containing the index info for all indexes
        /// on the specified table.
        /// </summary>
        /// <param name="tblname">the name of the table</param>
        /// <param name="tx">the calling transaction</param>
        /// <returns>a dictionary of IndexInfo objects, keyed by their field names</returns>
        public Dictionary<string, IndexInfo> getIndexInfo(string tblname, Transaction tx)
        {
            return idxmgr.getIndexInfo(tblname, tx);
        }

        /// <summary>
        /// Returns the statistical information about the specified table.
        /// </summary>
        /// <param name="tblname">the name of the table</param>
        /// <param name="ti">the table's metadata</param>
        /// <param name="tx">the calling transaction</param>
        /// <returns>the statistical information about the table</returns>
        public StatInfo getStatInfo(string tblname, TableInfo ti, Transaction tx)
        {
            return statmgr.getStatInfo(tblname, ti, tx);
        }
    }
}
