// migration from java to .NET by SimpleDB# group, 
// http://cs.tju.edu.cn/faculty/zhangkl/projects/simpledbsharp/index.htm

using System.Collections.Generic;
using simpledb.record;
using simpledb.tx;

namespace simpledb.metadata
{
    /// <summary>
    /// The statistics manager, which is responsible for
    /// keeping statistical information about each table.
    /// The manager does not store this information in the database.
    /// Instead, it calculates this information on system startup,
    /// and periodically refreshes it.
    /// </summary>
    /// <author>Edward Sciore</author>
    class StatMgr
    {
        private TableMgr tblMgr;
        private Dictionary<string, StatInfo> tablestats;
        private int numcalls;
        private object threadLock = new object();

        /// <summary>
        /// Creates the statistics manager.
        /// The initial statistics are calculated by
        /// traversing the entire database.
        /// </summary>
        /// <param name="tblMgr">The table manager.</param>
        /// <param name="tx">the startup transaction</param>
        public StatMgr(TableMgr tblMgr, Transaction tx)
        {
            this.tblMgr = tblMgr;
            refreshStatistics(tx);
        }

        /// <summary>
        /// Returns the statistical information about the specified table.
        /// </summary>
        /// <param name="tblname">the name of the table</param>
        /// <param name="ti">the table's metadata</param>
        /// <param name="tx">the calling transaction</param>
        /// <returns>the statistical information about the table</returns>
        public StatInfo getStatInfo(string tblname, TableInfo ti, Transaction tx)
        {
            lock (threadLock)
            {
                numcalls++;
                if (numcalls > 100)
                    refreshStatistics(tx);

                StatInfo si;
                if (!tablestats.ContainsKey(tblname))
                {
                    si = calcTableStats(ti, tx);
                    tablestats.Add(tblname, si);
                }
                else
                {
                    si = tablestats[tblname];
                }
                return si;
            }
        }

        private void refreshStatistics(Transaction tx)
        {
            lock (threadLock)
            {
                tablestats = new Dictionary<string, StatInfo>();
                numcalls = 0;
                TableInfo tcatmd = tblMgr.getTableInfo("tblcat", tx);
                RecordFile tcatfile = new RecordFile(tcatmd, tx);
                while (tcatfile.next())
                {
                    string tblname = tcatfile.getString("tblname");
                    TableInfo md = tblMgr.getTableInfo(tblname, tx);
                    StatInfo si = calcTableStats(md, tx);
                    tablestats.Add(tblname, si);
                }
                tcatfile.close();
            }
        }

        private StatInfo calcTableStats(TableInfo ti, Transaction tx)
        {
            lock (threadLock)
            {
                int numRecs = 0;
                RecordFile rf = new RecordFile(ti, tx);
                int numblocks = 0;
                while (rf.next())
                {
                    numRecs++;
                    numblocks = rf.currentRid().blockNumber()+1;
                }
                rf.close();
                return new StatInfo(numblocks, numRecs);
            }
        }
    }
}
