﻿// migration from java to .NET by SimpleDB# group, 
// http://cs.tju.edu.cn/faculty/zhangkl/projects/simpledbsharp/index.htm

using simpledb.tx;
using simpledb.record;
using simpledb.server;
using simpledb.query;
using simpledb.materialize;

namespace simpledb.multibuffer
{
    /// <summary>
    /// The Plan class for the muti-buffer version of the
    /// <i>product</i> operator.
    /// </summary>
    /// <author>Edward Sciore</author>
    public class MultiBufferProductPlan : Plan
    {
        private Plan lhs, rhs;
        private Transaction tx;
        private Schema sch = new Schema();

        /// <summary>
        ///  Creates a product plan for the specified queries.
        /// </summary>
        /// <param name="lhs">the plan for the LHS query</param>
        /// <param name="rhs">the plan for the RHS query</param>
        /// <param name="tx">the calling transaction</param>
        public MultiBufferProductPlan(Plan lhs, Plan rhs, Transaction tx)
        {
            this.lhs = lhs;
            this.rhs = rhs;
            this.tx = tx;
            sch.addAll(lhs.schema());
            sch.addAll(rhs.schema());
        }

        /// <summary>
        /// A scan for this query is created and returned, as follows.
        /// First, the method materializes its RHS query.
        /// It then determines the optimal chunk size,
        /// based on the size of the materialized file and the
        /// number of available buffers.
        /// It creates a chunk plan for each chunk, saving them in a list.
        /// Finally, it creates a multiscan for this list of plans,
        /// and returns that scan.
        /// </summary>
        /// <seealso cref="simpledb.query.Plan.open"/> 
        public Scan open()
        {
            TempTable tt = copyRecordsFrom(rhs);
            TableInfo ti = tt.getTableInfo();
            Scan leftscan = lhs.open();
            return new MultiBufferProductScan(leftscan, ti, tx);
        }

        /// <summary>
        /// Returns an estimate of the number of block accesses
        /// required to execute the query. The formula is:
        /// <code> B(product(p1,p2)) = B(p2) + B(p1)*C(p2) </code>
        /// where C(p2) is the number of chunks of p2.
        /// The method uses the current number of available buffers
        /// to calculate C(p2), and so this value may differ
        /// when the query scan is opened.
        /// </summary>
        /// <seealso cref="simpledb.query.Plan.blocksAccessed"/> 
        public int blocksAccessed()
        {
            // this guesses at the # of chunks
            int avail = SimpleDB.bufferMgr().available();
            int size = new MaterializePlan(rhs, tx).blocksAccessed();
            int numchunks = size / avail;
            return rhs.blocksAccessed() + (lhs.blocksAccessed() * numchunks);
        }

        /// <summary>
        /// Estimates the number of output records in the product.
        /// The formula is:
        /// <code> R(product(p1,p2)) = R(p1)*R(p2) </code>
        /// </summary>
        /// <seealso cref="simpledb.query.Plan.recordsOutput"/> 
        public int recordsOutput()
        {
            return lhs.recordsOutput() * rhs.recordsOutput();
        }

        /// <summary>
        /// Estimates the distinct number of field values in the product.
        /// Since the product does not increase or decrease field values,
        /// the estimate is the same as in the appropriate underlying query.
        /// </summary>
        /// <seealso cref=" simpledb.query.Plan.distinctValues"/>
        public int distinctValues(string fldname)
        {
            if (lhs.schema().hasField(fldname))
                return lhs.distinctValues(fldname);
            else
                return rhs.distinctValues(fldname);
        }

        /// <summary>
        /// /// Returns the schema of the product,
        /// which is the union of the schemas of the underlying queries.
        /// </summary> 
        /// <seealso cref="simpledb.query.Plan.schema"/> 
        public Schema schema()
        {
            return sch;
        }

        private TempTable copyRecordsFrom(Plan p)
        {
            Scan src = p.open();
            Schema sch = p.schema();
            TempTable tt = new TempTable(sch, tx);
            UpdateScan dest = (UpdateScan)tt.open();
            while (src.next())
            {
                dest.insert();
                foreach (string fldname in sch.fields())
                    dest.setVal(fldname, src.getVal(fldname));
            }
            src.close();
            dest.close();
            return tt;
        }
    }
}
