﻿// migration from java to .NET by SimpleDB# group, 
// http://cs.tju.edu.cn/faculty/zhangkl/projects/simpledbsharp/index.htm

using System.Collections.Generic;
using simpledb.record;

namespace simpledb.query
{
    /// <summary>
    /// /// A predicate is a Boolean combination of terms.
    /// </summary>
    /// <author>Edward Sciore</author>
    public class Predicate
    {
        private List<Term> terms = new List<Term>();

        /// <summary>
        ///  Creates an empty predicate.
        /// </summary>
        public Predicate() { }

        /// <summary>
        /// Creates a predicate containing a single term.
        /// </summary>
        /// <param name="T">the term</param>
        public Predicate(Term T)
        {
            terms.Add(T);
        }

        /// <summary>
        ///  Modifies the predicate to be the conjunction of
        /// itself and the specified predicate.
        /// </summary>
        /// <param name="pred">the other predicate</param> 
        public void conjoinWith(Predicate pred)
        {
            terms.AddRange(pred.terms);
        }

        /// <summary>
        /// Returns true if the predicate evaluates to true
        /// with respect to the specified scan.
        /// </summary>
        /// <param name="s">the scan</param> 
        /// <returns>true if the predicate is true in the scan</returns>
        public bool isSatisfied(Scan s)
        {
            foreach (Term t in terms)
                if (!t.isSatisfied(s))
                    return false;
            return true;
        }

        /// <summary>
        /// Calculates the extent to which selecting on the predicate 
        /// reduces the number of records output by a query.
        /// For example if the reduction factor is 2, then the
        /// predicate cuts the size of the output in half.
        /// </summary>
        /// <param name="p">the query's plan</param>  
        /// <returns>the integer reduction factor.</returns> 
        public int reductionFactor(Plan p)
        {
            int factor = 1;
            foreach (Term t in terms)
                factor *= t.reductionFactor(p);
            return factor;
        }

        /// <summary>
        /// Returns the subpredicate that applies to the specified schema.
        /// </summary>
        /// <param name="sch">the schema</param>
        /// <returns>the subpredicate applying to the schema</returns>
        public Predicate selectPred(Schema sch)
        {
            Predicate result = new Predicate();
            foreach (Term t in terms)
                if (t.appliesTo(sch))
                    result.terms.Add(t);
            if (result.terms.Count == 0)
                return null;
            else
                return result;
        }

        /// <summary>
        /// Returns the subpredicate consisting of terms that apply
        /// to the union of the two specified schemas, 
        /// but not to either schema separately.
        /// </summary>
        /// <param name="sch1">the first schema</param>
        /// <param name="sch2">the second schema</param>
        /// <returns>the subpredicate whose terms apply to the union of the two schemas but not either schema separately.</returns>
        public Predicate joinPred(Schema sch1, Schema sch2)
        {
            Predicate result = new Predicate();
            Schema newsch = new Schema();
            newsch.addAll(sch1);
            newsch.addAll(sch2);
            foreach (Term t in terms)
                if (!t.appliesTo(sch1) && !t.appliesTo(sch2) && t.appliesTo(newsch))
                    result.terms.Add(t);
            if (result.terms.Count == 0)
                return null;
            else
                return result;
        }

        /// <summary>
        /// Determines if there is a term of the form "F=c"
        /// where F is the specified field and c is some constant.
        /// If so, the method returns that constant.
        /// If not, the method returns null.
        /// </summary>
        /// <param name="fldname">the name of the field</param>
        /// <returns>either the constant or null</returns>
        public Constant equatesWithConstant(string fldname)
        {
            foreach (Term t in terms)
            {
                Constant c = t.equatesWithConstant(fldname);
                if (c != null)
                    return c;
            }
            return null;
        }

        /// <summary>
        /// Determines if there is a term of the form "F1=F2"
        /// where F1 is the specified field and F2 is another field.
        /// If so, the method returns the name of that field.
        /// If not, the method returns null.
        /// </summary>
        /// <param name="fldname">the name of the field</param>
        /// <returns>the name of the other field, or null</returns>
        public string equatsWithField(string fldname)
        {
            foreach (Term t in terms)
            {
                string s = t.equatesWithField(fldname);
                if (s != null)
                    return s;
            }
            return null;
        }

        /// <summary>
        /// Returns a <see cref="System.String"/> that represents this instance.
        /// </summary>
        /// <returns>
        /// A <see cref="System.String"/> that represents this instance.
        /// </returns>
        public override string ToString()
        {
            List<Term>.Enumerator iter = terms.GetEnumerator();
            if (!iter.MoveNext())
                return "";
            string result = iter.Current.ToString();
            while (iter.MoveNext())
                result += " and " + iter.Current.ToString();
            return result;
        }
    }
}
