﻿// migration from java to .NET by SimpleDB# group, 
// http://cs.tju.edu.cn/faculty/zhangkl/projects/simpledbsharp/index.htm

using simpledb.record;

namespace simpledb.query
{
    /// <summary>
    /// The Plan class corresponding to the <i>product</i>
    /// relational algebra operator.
    /// </summary>
    /// <author>Edward Sciore</author>
    public class ProductPlan : Plan
    {
        private Plan p1, p2;
        private Schema sch = new Schema();

        /// <summary>
        /// Creates a new product node in the query tree,
        /// having the two specified subqueries.
        /// </summary>
        /// <param name="p1">the left-hand subquery</param>
        /// <param name="p2">the right-hand subquery</param>
        public ProductPlan(Plan p1, Plan p2)
        {
            this.p1 = p1;
            this.p2 = p2;
            sch.addAll(p1.schema());
            sch.addAll(p2.schema());
        }

        /// <summary>
        ///  Creates a product scan for this query.
        /// </summary>
        /// <seealso cref="simpledb.query.Plan.open"/> 
        public Scan open()
        {
            Scan s1 = p1.open();
            Scan s2 = p2.open();
            return new ProductScan(s1, s2);
        }

        /// <summary>
        /// Estimates the number of block accesses in the product.
        /// The formula is:
        /// <code> B(product(p1,p2)) = B(p1) + R(p1)*B(p2) </code>
        /// </summary>
        /// <seealso cref="simpledb.query.Plan.blocksAccessed"/> 
        public int blocksAccessed()
        {
            return p1.blocksAccessed() + p1.recordsOutput() * p2.blocksAccessed();
        }

        /// <summary>
        /// Estimates the number of output records in the product.
        /// The formula is:
        /// <code> R(product(p1,p2)) = R(p1)*R(p2) </code>
        /// </summary>
        /// <seealso cref="simpledb.query.Plan.recordsOutput"/> 
        public int recordsOutput()
        {
            return p1.recordsOutput() * p2.recordsOutput();
        }

        /// <summary>
        ///  Estimates the distinct number of field values in the product.
        /// Since the product does not increase or decrease field values,
        /// the estimate is the same as in the appropriate underlying query.
        /// </summary>
        /// <seealso cref="simpledb.query.Plan.distinctValues"/> 
        public int distinctValues(string fldname)
        {
            if (p1.schema().hasField(fldname))
                return p1.distinctValues(fldname);
            else
                return p2.distinctValues(fldname);
        }

        /// <summary>
        /// Returns the schema of the product,
        /// which is the union of the schemas of the underlying queries.
        /// </summary>
        /// <seealso cref="simpledb.query.Plan.schema"/>
        public Schema schema()
        {
            return sch;
        }
    }
}
