﻿// migration from java to .NET by SimpleDB# group, 
// http://cs.tju.edu.cn/faculty/zhangkl/projects/simpledbsharp/index.htm


namespace simpledb.query
{
    /// <summary>
    /// The scan class corresponding to the <i>product</i> relational
    /// algebra operator.
    /// </summary>
    /// <author>Edward Sciore</author>
    public class ProductScan : Scan
    {
        private Scan s1, s2;

        /// <summary>
        /// Creates a product scan having the two underlying scans.
        /// </summary>
        /// <param name="s1">the LHS scan</param>
        /// <param name="s2">the RHS scan</param>
        public ProductScan(Scan s1, Scan s2)
        {
            this.s1 = s1;
            this.s2 = s2;
            s1.next();
        }

        /// <summary>
        ///  Positions the scan before its first record.
        /// In other words, the LHS scan is positioned at 
        /// its first record, and the RHS scan
        /// is positioned before its first record.
        /// </summary>
        /// <seealso cref="simpledb.query.Scan.beforeFirst"/> 
        public void beforeFirst()
        {
            s1.beforeFirst();
            s1.next();
            s2.beforeFirst();
        }

        /// <summary>
        /// Moves the scan to the next record.
        /// The method moves to the next RHS record, if possible.
        /// Otherwise, it moves to the next LHS record and the
        /// first RHS record.
        /// If there are no more LHS records, the method returns false.
        /// </summary>
        /// <seealso cref="simpledb.query.Scan.next"/> 
        public bool next()
        {
            if (s2.next())
                return true;
            else
            {
                s2.beforeFirst();
                return s2.next() && s1.next();
            }
        }

        /// <summary>
        /// Closes both underlying scans.
        /// </summary>
        /// /// <seealso cref="simpledb.query.Scan.close"/> 
        public void close()
        {
            s1.close();
            s2.close();
        }

        /// <summary>
        /// Returns the value of the specified field.
        /// The value is obtained from whichever scan
        /// contains the field.
        /// </summary>
        /// <seealso cref="simpledb.query.Scan.getVal"/>
        public Constant getVal(string fldname)
        {
            if (s1.hasField(fldname))
                return s1.getVal(fldname);
            else
                return s2.getVal(fldname);
        }

        /// <summary>
        /// Returns the integer value of the specified field.
        /// The value is obtained from whichever scan
        /// contains the field.
        /// </summary>
        /// <seealso cref="simpledb.query.Scan.getInt"/> 
        public int getInt(string fldname)
        {
            if (s1.hasField(fldname))
                return s1.getInt(fldname);
            else
                return s2.getInt(fldname);
        }

        /// <summary>
        ///  Returns the string value of the specified field.
        /// The value is obtained from whichever scan
        /// contains the field.
        /// </summary>
        /// <seealso cref="simpledb.query.Scan.getString"/> 
        public string getString(string fldname)
        {
            if (s1.hasField(fldname))
                return s1.getString(fldname);
            else
                return s2.getString(fldname);
        }

        /// <summary>
        /// Returns true if the specified field is in
        /// either of the underlying scans.
        /// </summary>
        /// <seealso cref="simpledb.query.Scan.hasField"/> 
        public bool hasField(string fldname)
        {
            return s1.hasField(fldname) || s2.hasField(fldname);
        }
    }
}
