﻿// migration from java to .NET by SimpleDB# group, 
// http://cs.tju.edu.cn/faculty/zhangkl/projects/simpledbsharp/index.htm

using System;
using simpledb.record;

namespace simpledb.query
{
    /// <summary>
    /// The Plan class corresponding to the <i>select</i>
    /// relational algebra operator.
    /// </summary> 
    /// <author>Edward Sciore</author>
    public class SelectPlan : Plan
    {
        private Plan p;
        private Predicate pred;

        /// <summary>
        /// Creates a new select node in the query tree,
        /// having the specified subquery and predicate.
        /// </summary>
        /// <param name="p">the subquery</param>
        /// <param name="pred">the predicate</param>
        public SelectPlan(Plan p, Predicate pred)
        {
            this.p = p;
            this.pred = pred;
        }

        /// <summary>
        ///  Creates a select scan for this query.
        /// </summary>
        /// <seealso cref="simpledb.query.Plan.open"/> 
        public Scan open()
        {
            Scan s = p.open();
            return new SelectScan(s, pred);
        }

        /// <summary>
        /// Estimates the number of block accesses in the selection,
        /// which is the same as in the underlying query.
        /// </summary> 
        /// <seealso cref="simpledb.query.Plan.blocksAccessed"/> 
        public int blocksAccessed()
        {
            return p.blocksAccessed();
        }

        /// <summary>
        ///  Estimates the number of output records in the selection,
        /// which is determined by the 
        /// reduction factor of the predicate.
        /// </summary>
        /// <seealso cref="simpledb.query.Plan.recordsOutput"/> 
        public int recordsOutput()
        {
            return p.recordsOutput() / pred.reductionFactor(p);
        }

        /// <summary>
        /// Estimates the number of distinct field values
        /// in the projection.
        /// If the predicate contains a term equating the specified 
        /// field to a constant, then this value will be 1.
        /// Otherwise, it will be the number of the distinct values
        /// in the underlying query 
        /// (but not more than the size of the output table).
        /// </summary>
        /// <seealso cref="simpledb.query.Plan.distinctValues"/> 
        public int distinctValues(string fldname)
        {
            if (pred.equatesWithConstant(fldname) != null)
                return 1;
            else
                return Math.Min(p.distinctValues(fldname), recordsOutput());
        }

        /// <summary>
        /// Returns the schema of the selection,
        /// which is the same as in the underlying query.
        /// </summary> 
        /// <seealso cref="simpledb.query.Plan.schema"/> 
        public Schema schema()
        {
            return p.schema();
        }
    }
}
