﻿// migration from java to .NET by SimpleDB# group, 
// http://cs.tju.edu.cn/faculty/zhangkl/projects/simpledbsharp/index.htm

using simpledb.record;

namespace simpledb.query
{
    /// <summary>
    /// The scan class corresponding to the <i>select</i> relational
    /// algebra operator.
    /// All methods except next delegate their work to the
    /// underlying scan.
    /// </summary> 
    /// <author>Edward Sciore</author>
    public class SelectScan : UpdateScan
    {
        private Scan s;
        private Predicate pred;

        /// <summary>
        /// Creates a select scan having the specified underlying
        /// scan and predicate.
        /// </summary>
        /// <param name="s">the scan of the underlying query</param>
        /// <param name="pred">the selection predicate</param>
        public SelectScan(Scan s, Predicate pred)
        {
            this.s = s;
            this.pred = pred;
        }

        /// <summary>
        ///  Positions the scan before its first record.
        /// </summary>
        public void beforeFirst()
        {
            s.beforeFirst();
        }

        /// <summary>
        ///  Move to the next record satisfying the predicate.
        /// The method repeatedly calls next on the underlying scan
        /// until a suitable record is found, or the underlying scan
        /// contains no more records.
        /// </summary>
        /// <seealso cref="simpledb.query.Scan.next"/>
        public bool next()
        {
            while (s.next())
                if (pred.isSatisfied(s))
                    return true;
            return false;
        }

        /// <summary>
        ///  Closes the scan and its subscans, if any. 
        /// </summary>
        public void close()
        {
            s.close();
        }

        /// <summary>
        /// Returns the value of the specified field in the current record.
        /// The value is expressed as a Constant.
        /// </summary>
        /// <param name="fldname">the name of the field</param>
        /// <returns>the value of that field, expressed as a Constant.</returns>
        public Constant getVal(string fldname)
        {
            return s.getVal(fldname);
        }

        /// <summary>
        /// Returns the value of the specified integer field 
        /// in the current record.
        /// </summary>
        /// <param name="fldname">the name of the field</param>
        /// <returns>the field's integer value in the current record</returns>
        public int getInt(string fldname)
        {
            return s.getInt(fldname);
        }

        /// <summary>
        ///  Returns the value of the specified string field 
        /// in the current record.
        /// </summary>
        /// <param name="fldname">the name of the field</param>
        /// <returns>the field's string value in the current record</returns>
        public string getString(string fldname)
        {
            return s.getString(fldname);
        }

        /// <summary>
        /// Returns true if the scan has the specified field.
        /// </summary>
        /// <param name="fldname">the name of the field</param>
        /// <returns>true if the scan has that field</returns>
        public bool hasField(string fldname)
        {
            return s.hasField(fldname);
        }

        /// <summary>
        ///  Modifies the field value of the current record.
        /// </summary>
        /// <param name="fldname"> the name of the field</param>
        /// <param name="val">the new value, expressed as a Constant</param>
        public void setVal(string fldname, Constant val)
        {
            UpdateScan us = (UpdateScan)s;
            us.setVal(fldname, val);
        }

        /// <summary>
        /// /// Modifies the field value of the current record.
        /// </summary>
        /// <param name="fldname">the name of the field</param>
        /// <param name="val">the new integer value</param>
        public void setInt(string fldname, int val)
        {
            UpdateScan us = (UpdateScan)s;
            us.setInt(fldname, val);
        }

        /// <summary>
        /// /// Modifies the field value of the current record.
        /// </summary>
        /// <param name="fldname">the name of the field</param>
        /// <param name="val">the new string value</param>
        public void setString(string fldname, string val)
        {
            UpdateScan us = (UpdateScan)s;
            us.setString(fldname, val);
        }

        /// <summary>
        /// Deletes the current record from the scan.
        /// </summary>
        public void delete()
        {
            UpdateScan us = (UpdateScan)s;
            us.delete();
        }

        /// <summary>
        /// Inserts a new record somewhere in the scan.
        /// </summary>
        public void insert()
        {
            UpdateScan us = (UpdateScan)s;
            us.insert();
        }

        /// <summary>
        ///  Returns the RID of the current record.
        /// </summary>
        /// <returns>the RID of the current record</returns>
        public RID getRid()
        {
            UpdateScan us = (UpdateScan)s;
            return us.getRid();
        }

        /// <summary>
        ///  Positions the scan so that the current record has
        /// the specified RID.
        /// </summary>
        /// <param name="rid">the RID of the desired record</param>
        public void moveToRid(RID rid)
        {
            UpdateScan us = (UpdateScan)s;
            us.moveToRid(rid);
        }
    }
}
