﻿// migration from java to .NET by SimpleDB# group, 
// http://cs.tju.edu.cn/faculty/zhangkl/projects/simpledbsharp/index.htm

using simpledb.tx;
using simpledb.record;

namespace simpledb.query
{
    /// <summary>
    /// The Scan class corresponding to a table.
    /// A table scan is just a wrapper for a RecordFile object;
    /// most methods just delegate to the corresponding
    /// RecordFile methods.
    /// </summary>
    /// <author>Edward Sciore</author>
    public class TableScan : UpdateScan
    {
        private RecordFile rf;
        private Schema sch;

        /// <summary>
        /// Creates a new table scan,
        /// and opens its corresponding record file. 
        /// </summary>
        /// <param name="ti">the table's metadata</param>
        /// <param name="tx">the calling transaction</param>
        public TableScan(TableInfo ti, Transaction tx)
        {
            rf = new RecordFile(ti, tx);
            sch = ti.schema();
        }

        /// <summary>
        ///  Positions the scan before its first record.
        /// </summary>
        public void beforeFirst()
        {
            rf.beforeFirst();
        }

        /// <summary>
        /// Moves the scan to the next record.
        /// </summary>
        /// <returns>false if there is no next record</returns> 
        public bool next()
        {
            return rf.next();
        }

        /// <summary>
        ///  Closes the scan and its subscans, if any. 
        /// </summary>
        public void close()
        {
            rf.close();
        }

        /// <summary>
        ///  Returns the value of the specified field, as a Constant.
        /// The schema is examined to determine the field's type.
        /// If INTEGER, then the record file's getInt method is called;
        /// otherwise, the getString method is called.
        /// </summary> 
        /// <seealso cref="simpledb.query.Scan.getVal"/> 
        public Constant getVal(string fldname)
        {
            if (sch.type(fldname) == Schema.INTEGER)
                return new IntConstant(rf.getInt(fldname));
            else
                return new StringConstant(rf.getString(fldname));
        }

        /// <summary>
        /// Returns the value of the specified integer field 
        /// in the current record.
        /// </summary>
        /// <param name="fldname">the name of the field</param>
        /// <returns>the field's integer value in the current record</returns>
        public int getInt(string fldname)
        {
            return rf.getInt(fldname);
        }

        /// <summary>
        ///  Returns the value of the specified string field 
        /// in the current record.
        /// </summary>
        /// <param name="fldname">the name of the field</param>
        /// <returns>the field's string value in the current record</returns>
        public string getString(string fldname)
        {
            return rf.getString(fldname);
        }

        /// <summary>
        /// Returns true if the scan has the specified field.
        /// </summary>
        /// <param name="fldname">the name of the field</param>
        /// <returns>true if the scan has that field</returns>
        public bool hasField(string fldname)
        {
            return sch.hasField(fldname);
        }

        /// <summary>
        ///  Sets the value of the specified field, as a Constant.
        /// The schema is examined to determine the field's type.
        /// If INTEGER, then the record file's setInt method is called;
        /// otherwise, the setString method is called.
        /// </summary>
        /// <seealso cref="simpledb.query.UpdateScan.setVal"/> 
        public void setVal(string fldname, Constant val)
        {
            if (sch.type(fldname) == Schema.INTEGER)
                rf.setInt(fldname, (int)val.asCsharpVal());
            else
                rf.setString(fldname, (string)val.asCsharpVal());
        }

        /// <summary>
        /// /// Modifies the field value of the current record.
        /// </summary>
        /// <param name="fldname">the name of the field</param>
        /// <param name="val">the new string value</param>
        public void setString(string fldname, string val)
        {
            rf.setString(fldname, val);
        }

        /// <summary>
        /// /// Modifies the field value of the current record.
        /// </summary>
        /// <param name="fldname">the name of the field</param>
        /// <param name="val">the new integer value</param>
        public void setInt(string fldname, int val)
        {
            rf.setInt(fldname, val);
        }

        /// <summary>
        /// Deletes the current record from the scan.
        /// </summary>
        public void delete()
        {
            rf.delete();
        }

        /// <summary>
        /// Inserts a new record somewhere in the scan.
        /// </summary>
        public void insert()
        {
            rf.insert();
        }

        /// <summary>
        ///  Returns the RID of the current record.
        /// </summary>
        /// <returns>the RID of the current record</returns>
        public RID getRid()
        {
            return rf.currentRid();
        }

        /// <summary>
        ///  Positions the scan so that the current record has
        /// the specified RID.
        /// </summary>
        /// <param name="rid">the RID of the desired record</param>
        public void moveToRid(RID rid)
        {
            rf.moveToRid(rid);
        }
    }

}
