﻿// migration from java to .NET by SimpleDB# group, 
// http://cs.tju.edu.cn/faculty/zhangkl/projects/simpledbsharp/index.htm

using System;
using simpledb.record;

namespace simpledb.query
{
    /// <summary>
    /// A term is a comparison between two expressions.
    /// </summary>
    /// <author>Edward Sciore</author>
    public class Term
    {
        private Expression lhs, rhs;

        /// <summary>
        /// Creates a new term that compares two expressions
        /// for equality.
        /// </summary>
        /// <param name="lhs">the LHS expression</param>
        /// <param name="rhs">the RHS expression</param>
        public Term(Expression lhs, Expression rhs)
        {
            this.lhs = lhs;
            this.rhs = rhs;
        }

        /// <summary>
        ///  Calculates the extent to which selecting on the term reduces 
        /// the number of records output by a query.
        /// For example if the reduction factor is 2, then the
        /// term cuts the size of the output in half.
        /// </summary>
        /// <param name="p">the query's plan</param>
        /// <returns>the integer reduction factor.</returns>
        public int reductionFactor(Plan p)
        {
            string lhsName, rhsName;
            if (lhs.isFieldName() && rhs.isFieldName())
            {
                lhsName = lhs.asFieldName();
                rhsName = rhs.asFieldName();
                return Math.Max(p.distinctValues(lhsName), p.distinctValues(rhsName));
            }

            if (lhs.isFieldName())
            {
                lhsName = lhs.asFieldName();
                return p.distinctValues(lhsName);
            }

            if (rhs.isFieldName())
            {
                rhsName = rhs.asFieldName();
                return p.distinctValues(rhsName);
            }
            
            // otherwise, the term equates constants
            if (lhs.asConstant().Equals(rhs.asConstant()))
                return 1;
            else
                return int.MaxValue;
        }

        /// <summary>
        /// Determines if this term is of the form "F=c"
        /// where F is the specified field and c is some constant.
        /// If so, the method returns that constant.
        /// If not, the method returns null.
        /// </summary>
        /// <param name="fldname">fldname the name of the field</param>
        /// <returns>either the constant or null</returns>
        public Constant equatesWithConstant(string fldname)
        {
            if (lhs.isFieldName() && lhs.asFieldName().Equals(fldname) && rhs.isConstant())
                return rhs.asConstant();
            else if (rhs.isFieldName() && rhs.asFieldName().Equals(fldname) && lhs.isConstant())
                return lhs.asConstant();
            else
                return null;
        }

        /// <summary>
        /// Determines if this term is of the form "F1=F2"
        /// where F1 is the specified field and F2 is another field.
        /// If so, the method returns the name of that field.
        /// If not, the method returns null.
        /// </summary>
        /// <param name="fldname">fldname the name of the field</param>
        /// <returns>either the name of the other field, or null</returns>
        public string equatesWithField(string fldname)
        {
            if (lhs.isFieldName() && lhs.asFieldName().Equals(fldname) && rhs.isFieldName())
                return rhs.asFieldName();
            else if (rhs.isFieldName() && rhs.asFieldName().Equals(fldname) && lhs.isFieldName())
                return lhs.asFieldName();
            else
                return null;
        }

        /// <summary>
        /// Returns true if both of the term's expressions
        /// apply to the specified schema. 
        /// </summary>
        /// <param name="sch">the schema</param>
        /// <returns>true if both expressions apply to the schema</returns>
        public bool appliesTo(Schema sch)
        {
            return lhs.appliesTo(sch) && rhs.appliesTo(sch);
        }

        /// <summary>
        /// Returns true if both of the term's expressions
        /// evaluate to the same constant,
        /// with respect to the specified scan.
        /// </summary>
        /// <param name="s">the scan</param>
        /// <returns>true if both expressions have the same value in the scan</returns>
        public bool isSatisfied(Scan s)
        {
            Constant lhsval = lhs.evaluate(s);
            Constant rhsval = rhs.evaluate(s);
            return rhsval.Equals(lhsval);
        }

        /// <summary>
        /// Returns a <see cref="System.String"/> that represents this instance.
        /// </summary>
        /// <returns>
        /// A <see cref="System.String"/> that represents this instance.
        /// </returns>
        public override string ToString()
        {
            return lhs.ToString() + "=" + rhs.ToString();
        }
    }
}
