﻿// migration from java to .NET by SimpleDB# group, 
// http://cs.tju.edu.cn/faculty/zhangkl/projects/simpledbsharp/index.htm

using System.Collections.Generic;

namespace simpledb.record
{
    /// <summary>
    /// The record schema of a table.
    /// A schema contains the name and type of
    /// each field of the table, as well as the length
    /// of each varchar field.
    /// </summary>
    /// <author>Edward Sciore</author>
    public class Schema
    {
        private Dictionary<string, FieldInfo> info = new Dictionary<string, FieldInfo>();

        /// <summary>
        /// type represented by int constant, values are from JDBC
        /// </summary>
        public const int INTEGER = 4, VARCHAR = 12;

        /// <summary>
        /// Creates an empty schema.
        /// Field information can be added to a schema
        /// via the five addXXX methods. 
        /// </summary>
        public Schema() { }

        /// <summary>
        /// Adds a field to the schema having a specified
        /// name, type, and length.
        /// If the field type is "integer", then the length
        /// value is irrelevant.. 
        /// </summary>
        /// <param name="fldname">the name of the field</param>
        /// <param name="type">the type of the field, according to the constants in simpledb.sql.types</param>
        /// <param name="length">the conceptual length of a string field</param>
        public void addFiled(string fldname, int type, int length)
        {
            info.Add(fldname, new FieldInfo(type, length));
        }

        /// <summary>
        /// Adds an integer field to the schema.
        /// </summary>
        /// <param name="fldname">the name of the field</param>
        public void addIntField(string fldname)
        {
            addFiled(fldname, INTEGER, 0);
        }

        /// <summary>
        /// Adds a string field to the schema.
        /// The length is the conceptual length of the field.
        /// For example, if the field is defined as varchar(8),
        /// then its length is 8.
        /// </summary>
        /// <param name="fldname">the name of the field</param>
        /// <param name="length">the number of chars in the varchar definition</param>
        public void addStringField(string fldname, int length)
        {
            addFiled(fldname, VARCHAR, length);
        }

        /// <summary>
        /// Adds a field to the schema having the same
        /// type and length as the corresponding field
        /// in another schema.
        /// </summary>
        /// <param name="fldname">the name of the field</param>
        /// <param name="sch">the other schema</param>
        public void add(string fldname, Schema sch)
        {
            int type = sch.type(fldname);
            int length = sch.length(fldname);
            addFiled(fldname, type, length);
        }

        /// <summary>
        /// Adds all of the fields in the specified schema
        /// to the current schema.
        /// </summary>
        /// <param name="sch">the other schema</param>
        public void addAll(Schema sch)
        {
            foreach (KeyValuePair<string, FieldInfo> kvp in sch.info)
                info.Add(kvp.Key, kvp.Value);
        }

        /// <summary>
        /// Returns a collection containing the name of
        /// each field in the schema.
        /// </summary>
        /// <returns>the collection of the schema's field names</returns>
        public ICollection<string> fields()
        {
            return info.Keys;
        }

        /// <summary>
        /// Returns true if the specified field
        /// is in the schema
        /// </summary>
        /// <param name="fldname">the name of the field</param>
        /// <returns>true if the field is in the schema</returns>
        public bool hasField(string fldname)
        {
            return info.ContainsKey(fldname);
        }

        /// <summary>
        /// Returns the type of the specified field, using the
        /// constants..
        /// </summary>
        /// <param name="fldname">the name of the field</param>
        /// <returns>the integer type of the field</returns>
        public int type(string fldname)
        {
            return info[fldname].type;
        }

        /// <summary>
        /// Returns the conceptual length of the specified field.
        /// If the field is not a string field, then
        /// the return value is undefined.
        /// </summary>
        /// <param name="fldname">the name of the field</param>
        /// <returns>the conceptual length of the field</returns>
        public int length(string fldname)
        {
            return info[fldname].length;
        }

        private class FieldInfo
        {
            public int type, length;
            public FieldInfo(int type, int length)
            {
                this.type = type;
                this.length = length;
            }
        }
    }
}
