// migration from java to .NET by SimpleDB# group, 
// http://cs.tju.edu.cn/faculty/zhangkl/projects/simpledbsharp/index.htm

using simpledb.file;
using simpledb.log;
using simpledb.buffer;
using simpledb.tx;
using simpledb.metadata;
using simpledb.planner;
using simpledb.index.planner;
using simpledb.opt;

namespace simpledb.server
{
    /// <summary>
    /// The class that provides system-wide static global values.
    /// These values must be initialized by the method
    /// <see cref="init"/> before use.
    /// The methods <see cref="initFileMgr"/>,
    /// <see cref="initFileAndLogMgr"/>,
    /// <see cref="initFileLogAndBufferMgr"/>,
    /// and <see cref="initMetadataMgr"/>
    /// provide limited initialization, and are useful for 
    /// debugging purposes.
    /// </summary>
    /// <author>Edward Sciore</author>
    public class SimpleDB
    {
        /// <summary>
        /// Default buffer size
        /// </summary>
        public static int BUFFER_SIZE = 8;

        /// <summary>
        /// Default log file name
        /// </summary>
        public static string LOG_FILE = "simpledb.log";

        private static FileMgr fm;
        private static LogMgr logm;
        private static BufferMgr bm;
        private static MetadataMgr mdm;

        /// <summary>
        /// Initializes the system.
        /// This method is called during system startup.
        /// </summary>
        /// <param name="dirname">the name of the database directory</param>
        public static void init(string dirname)
        {
            initFileLogAndBufferMgr(dirname);
            Transaction tx = new Transaction();
            bool isnew = fm.isNew();
            if (isnew)
                System.Console.WriteLine("creating new database");
            else
            {
                System.Console.WriteLine("recovering existing database");
                tx.recover();
            }
            initMetadataMgr(isnew, tx);
            tx.commit();
        }

        // The following initialization methods are useful for 
        // testing the lower-level components of the system 
        // without having to initialize everything.

        /// <summary>
        /// Initializes only the file manager.
        /// </summary>
        /// <param name="dirname">dirname the name of the database directory</param>
        public static void initFileMgr(string dirname)
        {
            fm = new FileMgr(dirname);
        }

        /// <summary>
        /// Initializes the file and log managers.
        /// </summary>
        /// <param name="dirname">the name of the database directory</param>
        public static void initFileAndLogMgr(string dirname)
        {
            initFileMgr(dirname);
            logm = new LogMgr(LOG_FILE);
        }

        /// <summary>
        /// Initializes the file, log, and buffer managers.
        /// </summary>
        /// <param name="dirname">the name of the database directory</param>
        public static void initFileLogAndBufferMgr(string dirname)
        {
            initFileAndLogMgr(dirname);
            bm = new BufferMgr(BUFFER_SIZE);
        }

        /// <summary>
        /// Initializes metadata manager.
        /// </summary>
        /// <param name="isnew">an indication of whether a new database needs to be created.</param>
        /// <param name="tx">the transaction performing the initialization</param>
        public static void initMetadataMgr(bool isnew, Transaction tx)
        {
            mdm = new MetadataMgr(isnew, tx);
        }

        /// <summary>
        /// Return the SimpleDB File Manager
        /// </summary>
        /// <returns>the FileMgr</returns>
        public static FileMgr fileMgr() { return fm; }

        /// <summary>
        /// Return the SimpleDB Log Manager
        /// </summary>
        /// <returns>the LogMgr</returns>
        public static LogMgr logMgr() { return logm; }

        /// <summary>
        /// Return the SimpleDB Buffer Manager
        /// </summary>
        /// <returns>the BufferMgr</returns>
        public static BufferMgr bufferMgr() { return bm; }

        /// <summary>
        /// Return the SimpleDB metadata Manager
        /// </summary>
        /// <returns>the MeatadataMgr</returns>
        public static MetadataMgr mdMgr() { return mdm; }


        /// <summary>
        /// Creates a planner for SQL commands.
        /// To change how the planner works, modify this method.
        /// </summary>
        /// <returns>the system's planner for SQL commands</returns>
        public static Planner planner()
        {
            //QueryPlanner qplanner = new BasicQueryPlanner();
            //UpdatePlanner uplanner = new BasicUpdatePlanner();
            QueryPlanner qplanner = new HeuristicQueryPlanner();
            UpdatePlanner uplanner = new IndexUpdatePlanner();
            return new Planner(qplanner, uplanner);
        }
    }
}
