// migration from java to .NET by SimpleDB# group, 
// http://cs.tju.edu.cn/faculty/zhangkl/projects/simpledbsharp/index.htm

using System.Collections.Generic;
using simpledb.file;
using simpledb.buffer;
using simpledb.server;

namespace simpledb.tx.recovery
{
    /// <summary>
    /// The recovery manager. Each transaction has its own recovery manager.
    /// </summary>
    /// <author>Edward Sciore</author>
    class RecoveryMgr
    {
        private int txnum;

        /// <summary>
        /// Creates a recovery manager for the specified transaction.
        /// </summary>
        /// <param name="txnum">the ID of the specified transaction</param>
        public RecoveryMgr(int txnum)
        {
            this.txnum = txnum;
            new StartRecord(txnum).writeToLog();
        }

        /// <summary>
        /// Writes a commit record to the log, and flushes it to disk 
        /// </summary>
        public void commit()
        {
            SimpleDB.bufferMgr().flushAll(txnum);
            int lsn = new CommitRecord(txnum).writeToLog();
            SimpleDB.logMgr().flush(lsn);
        }

        /// <summary>
        /// Writes a rollback record to the log, and flushes it to disk 
        /// </summary>
        public void rollback()
        {
            SimpleDB.bufferMgr().flushAll(txnum);
            doRollback();
            int lsn = new RollbackRecord(txnum).writeToLog();
            SimpleDB.logMgr().flush(lsn);
        }

        /// <summary>
        /// Recovers uncompleted transactions from the log,
        /// then writes a quiescent checkpoint record to the log and flushes it 
        /// </summary>
        public void recover()
        {
            doRecover();
            int lsn = new CheckpointRecord().writeToLog();
            SimpleDB.logMgr().flush(lsn);
        }

        /// <summary>
        /// Writes a setint record to the log, and returns its lsn.
        /// Updates to temporary files are not logged; instead, a
        /// "dummy" negative lsn is returned.
        /// </summary>
        /// <param name="buff">the buffer containing the page</param>
        /// <param name="offset">the offset of the value in the page</param>
        /// <param name="newval">the value to be written</param>
        /// <returns>the lsn of SETINT log record</returns>
        public int setInt(Buffer buff, int offset, int newval)
        {
            int oldval = buff.getInt(offset);
            Block blk = buff.block();
            if (isTempBlock(blk))
                return -1;
            else
                return new SetIntRecord(txnum, blk, offset, oldval).writeToLog();
        }

        /// <summary>
        /// Writes a setString record to the log, and returns its lsn.
        /// Updates to temporary files are not logged; instead, a
        /// "dummy" negative lsn is returned.
        /// </summary>
        /// <param name="buff">the buffer containing the page</param>
        /// <param name="offset">the offset of the value in the page</param>
        /// <param name="newval">the value to be written</param>
        /// <returns>the lsn of SETSTRING log record</returns>
        public int setString(Buffer buff, int offset, string newval)
        {
            string oldval = buff.getString(offset);
            Block blk = buff.block();
            if (isTempBlock(blk))
                return -1;
            else
                return new SetStringRecord(txnum, blk, offset, oldval).writeToLog();
        }

        /// <summary>
        /// Rolls back the transaction.
        /// The method iterates through the log records,
        /// calling undo() for each log record it finds
        /// for the transaction,
        /// until it finds the transaction's START record. 
        /// </summary>
        private void doRollback()
        {

            LogRecordIterator iter = new LogRecordIterator();
            while (iter.MoveNext())
            {
                LogRecord rec = iter.Current;
                if (rec.txNumber() == txnum)
                {
                    if (rec.op() == LogRecord.START)
                        return;
                    rec.undo(txnum);
                }
            }
        }

        /// <summary>
        /// Does a complete database recovery.
        /// The method iterates through the log records.
        /// Whenever it finds a log record for an unfinished
        /// transaction, it calls undo() on that record.
        /// The method stops when it encounters a CHECKPOINT record
        /// or the end of the log 
        /// </summary>
        private void doRecover()
        {
            List<int> finishedTxs = new List<int>();
            LogRecordIterator iter = new LogRecordIterator();
            while (iter.MoveNext())
            {
                LogRecord rec = iter.Current;
                if (rec.op() == LogRecord.CHECKPOINT)
                    return;
                if (rec.op() == LogRecord.COMMIT || rec.op() == LogRecord.ROLLBACK)
                    finishedTxs.Add(rec.txNumber());
                else if (!finishedTxs.Contains(rec.txNumber()))
                    rec.undo(txnum);
            }
        }

        /// <summary>
        /// Determines whether a block comes from a temporary file or not 
        /// </summary>
        /// <param name="blk">a block</param>
        /// <returns>true if the block comes from a temporary file, other false</returns>
        private bool isTempBlock(Block blk)
        {
            return blk.fileName().StartsWith("temp");
        }
    }
}
